require "thor"

module C42
  class C42file < ::Thor
    require "tempfile"
    include Thor::Actions

    no_commands {
      def shell
        @shell ||= Thor::Base.shell.new
      end

      def error(msg)
        shell.say_status("ERROR", msg, :red)
      end

      def warn(msg)
        shell.say_status("WARN", msg, :yellow)
      end

      def info(msg)
        shell.say_status("INFO", msg, :on_black)
      end

      def debug(msg)
        shell.say_status("DEBUG", msg, :green)
      end

      def fatal(msg)
        shell.say_status("FATAL", msg, :on_red)
        exit
      end
    }

    class << self
      @@pkg_name = nil
      @@use_cwd = nil
      @@file_pwd = %x{pwd}
      @@stop_here = false

      alias_method :__original_desc, :desc
      def desc(name, *args)
        self.send(:__original_desc, *(["#{@@pkg_name}#{name}"] | (args || [])))
      end

      def use_cwd(&block)
        @@use_cwd = true
        yield
        @@use_cwd = nil
      end


      def task(name, &block)
        use_cwd = @@use_cwd
        file_pwd = @@file_pwd
        self.instance_eval do
          define_method :"#{@@pkg_name}#{name}" do |*args|
            Dir.chdir(use_cwd ? Dir.pwd : file_pwd) do
              instance_exec *args, &block
            end
          end
        end
      end

      def shell_task(name, command, options = {})
        options = {message: :system, recipient: Kernel, use_cwd: false}.merge(options || {})
        task :"#{name}" do |*args|
          cmd = ([command] | (args || [])).flatten
          (options[:recipient]).send(
            options[:message], 
            *(
              command.is_a?(String) ? [cmd.join(" ")] : cmd
            )
          )
        end
      end

      def package(pkg_name, &block)
        self.instance_eval do
          @@pkg_name = "#{pkg_name}:"
          yield  
          @@pkg_name = nil
        end
      end

      def stop_here!
        @@stop_here = true
      end

      def stopped_here?
        @@stop_here
      end
    end
  end
end
